// Copyright (C) 1997, 1998 by DI Wolfgang Platzer, IAIK
// email: wplatzer@iaik.tu-graz.ac.at
//
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.
//

package iaik.jigsaw.https;

import java.net.*;
import java.security.*;
import java.security.cert.X509Certificate;

import org.w3c.jigsaw.daemon.*;
import org.w3c.util.*;
import org.w3c.jigsaw.http.*;
import org.w3c.jigsaw.http.socket.SSLSocketClient;
import org.w3c.tools.resources.*;
import org.w3c.tools.resources.ProtocolException;

import iaik.security.ssl.*;

/**
 * This is the main class of the SSL server. It subclasses the default
 * Jigsaw server and adds the SSL functionality.
 *
 * @see SSLSocketClientFactory
 * @see SSLSocketClient
 *
 * @autor wplatzer
 */
public class httpsd extends org.w3c.jigsaw.http.httpd {

  // for default is SSL disabled
  protected boolean ssl_enabled = false;

  /**
   * This property must be set to "true" to enable SSL.
   */
  public static final String 
  SSL_ENABLED_P = "iaik.jigsaw.ssl.enabled";

  /**
   * Specifies the security provider which shall be used.
   * This value defaults to "iaik.security.provider.IAIK"
   */
  public static final String 
  SECURITY_PROVIDER_P = "iaik.jigsaw.ssl.security.provider";

  /**
   * Initialize the SSL properties of this server.
   */
  protected void initializeProperties() throws ServerHandlerInitException {
  
    // first let the super class initialize its properties
    super.initializeProperties();

    // is SSL enabled ?
    ssl_enabled = props.getBoolean(SSL_ENABLED_P, false);
    
    if (ssl_enabled) {
      // if SSL is enabled we need a security provider
      
      String provider_name = props.getString(SECURITY_PROVIDER_P, "iaik.security.provider.IAIK");
      try {
        Class provider_class = Class.forName(provider_name);
        Provider provider = (Provider)provider_class.newInstance();
        Security.addProvider(provider);
        System.out.println("\nProvider: "+provider.toString()+" added.");
      } catch (Exception ex) {
        errlog("Unable to set provider - SSL disabled.");
        System.out.println("Unable to set provider - SSL disabled.");
        ssl_enabled = false;
      }
    }
  }
 
  public ReplyInterface perform(RequestInterface req) throws ProtocolException, ResourceException {
    Request request = (Request)req;
    Client client = request.getClient();
    if (client instanceof SSLSocketClient && ((SSLSocketClient)client).isSSLConnection()) {
      // set request protocol to https
      
      URL url = request.getURL();
      try {
        request.setURL(new URL("https", url.getHost(), url.getPort(), url.getFile()));
      } catch (MalformedURLException ex) {
        errlog("MalformedURLException while changing protocol to https.");
      }

      // Set properties for Servlets
      String cs = ((SSLSocketClient)client).getActiveCipherSuite().getName();
      request.setState("javax.net.ssl.cipher_suite", cs);
      String cm = ((SSLSocketClient)client).getActiveCompressionMethod().getName();
      request.setState("javax.net.ssl.compression_method", cm);
      X509Certificate[] clientCertChain = ((SSLSocketClient)client).getClientCertificateChain();
      request.setState("javax.net.ssl.peer_certificates", clientCertChain);
    }
    
    return super.perform(req);
  }
  
  /**
   * Creates a new httpsd.
   */
  public httpsd() {
  	super();
  	
  	// Handler factory for https
  	URL.setURLStreamHandlerFactory(new iaik.protocol.https.HandlerFactory());
  }

  /**
   * Returns true if SSL is enabled.
   *
   * @return true if SSL is enabled
   */
  public boolean sslEnabled() {
    return ssl_enabled;
  }
  
 /**
  * Returns the server URL.
  *
  * @return the server URL
  */
  public URL getURL() {
    if (!ssl_enabled)
  		return super.getURL();

    if (url == null) {
	    try {
    		if (port != 443) 
  		    url = new URL("https", host, port, "/");
	    	else
		      url = new URL("https", host, "/");
	    } catch (MalformedURLException ex) {
    		throw new RuntimeException("unable to build server's URL");
	    }
	  }		
	  return url;
	}
	
	protected String getBanner() {
	  return "Jigsaw-SSL["+version+"]";
	}
}